/* CG/solution/parts/cgp01.c */

/****************************************************************
 *                                                              *
 * This file has been written as a sample solution to an        *
 * exercise in a course given at the High Performance           *
 * Computing Centre Stuttgart (HLRS).                           *
 * It is made freely available with the understanding that      *
 * every copy of this file must include this header and that    *
 * HLRS takes no responsibility for the use of the enclosed     *
 * teaching material.                                           *
 *                                                              *
 * This software is partitioned into 16 tasks.                  *
 * Each task can be implemented by one group of the course      *
 * Files: CG/solution/cg_all.c - Solution                       *
 *        CG/skel/cg_01.c      - Skeletons that should be used  *
 *             .. cg_15.c        to implement each task         *
 *        CG/solution/cgp00.c  - Source part of each task       *
 *             ..     cgp16.c                                   *
 *                                                              *
 * Authors: Dr. Rolf Rabenseifner                               *
 *          Felix Triebel                                       *
 *          Michael Speck                                       *
 *          Felix Triebel                                       *
 *          Traugott Streicher                                  *
 *                                                              *
 * Contact: rabenseifner@hlrs.de                                *
 *                                                              *
 ****************************************************************/
                                                           /*EOH*/
/*>>>>>>>>>>>>>>>>>>>>>> begin of task 01 <<<<<<<<<<<<<<<<<<<<<<*/

/*********************** distribution ************************/

/*------------------------------
 * void Distribution
 *  IN  int mat_size : size of the matrix
 *
 * TASK: The heat matrix is distributed in slices of equal size maybe except for
 *       the last process. In this program, the slices are called chunks.
 *       This function calculates the chunk_size, the start and the end1
 *       value for the local chunk. It uses the size of the matrix, the number
 *       of processes and the rank for this calculation.
 *
 * REFERENCE: Slides 13, 15, 24, 26
 *
 * USED GLOBAL DATA:
 *  IN  int my_rank    : rank of the local process (my_rank)
 *  IN  int numprocs   : number of processes
 *  OUT int chunk_size : size of the local chunk
 *  OUT int start      : index for the beginning of the local chunk
 *  OUT int end1       : index+1 for the end of the local chunk
 * USED FUNCTIONS: none
 */
void Distribution(int mat_size)
{
        chunk_size=(mat_size-1)/numprocs + 1; // rounding up
        start = chunk_size*my_rank;
        if (start > mat_size)  start = mat_size;
        end1 = chunk_size*(my_rank+1);
        if (end1  > mat_size)  end1 = mat_size;
}


/*------------------------------
 * int Row2Rank
 *  IN int     : global_index
 *  RETURNS int: rank
 *
 * TASK: Given a matrix or vector global row index, it calculates
 *       the processor's rank on which it should be located.
 *
 * REFERENCE: see Distribution() above
 *
 * GLOBAL DATA:
 *  IN int chunk_size : size of the local chunk
 * USED FUNCTIONS: none
 */
int Row2Rank(int global_index)
{
       return (global_index / chunk_size);
}

/*------------------------------
 * int Rank2StartRow
 *  IN int     : rank
 *  RETURNS int: global start index
 *
 * TASK: Calculates the first global row index for a given rank
 *
 * REFERENCE: see Distribution() above
 *
 * GLOBAL DATA:
 *  IN int chunk_size : size of the local chunk
 * USED FUNCTIONS: none
 */
int Rank2StartRow(int rank)
{
        return(chunk_size*rank);
}

/*>>>>>>>>>>>>>>>>>>>>>> -end- of task 01 <<<<<<<<<<<<<<<<<<<<<<*/
