/* CG/solution/parts/cgp12.c */

/****************************************************************
 *                                                              *
 * This file has been written as a sample solution to an        *
 * exercise in a course given at the High Performance           *
 * Computing Centre Stuttgart (HLRS).                           *
 * It is made freely available with the understanding that      *
 * every copy of this file must include this header and that    *
 * HLRS takes no responsibility for the use of the enclosed     *
 * teaching material.                                           *
 *                                                              *
 * This software is partitioned into 16 tasks.                  *
 * Each task can be implemented by one group of the course      *
 * Files: CG/solution/cg_all.c - Solution                       *
 *        CG/skel/cg_01.c      - Skeletons that should be used  *
 *             .. cg_15.c        to implement each task         *
 *        CG/solution/cgp00.c  - Source part of each task       *
 *             ..     cgp16.c                                   *
 *                                                              *
 * Authors: Dr. Rolf Rabenseifner                               *
 *          Felix Triebel                                       *
 *          Michael Speck                                       *
 *          Felix Triebel                                       *
 *          Traugott Streicher                                  *
 *                                                              *
 * Contact: rabenseifner@hlrs.de                                *
 *                                                              *
 ****************************************************************/
                                                           /*EOH*/
/*>>>>>>>>>>>>>>>>>>>>>> begin of task 12 <<<<<<<<<<<<<<<<<<<<<<*/

/*------------------------------
 * void Init_Boundary_Vec
 *  OUT vector_chunk b: the local chunk of vector b
 *
 * TASK: Store all boundary values in the vector memory 
 *       (that is already allocated).
 *       In this example, the boundaries are given by:
 *         u(i,j) = h * (i+1)  with  h = 1.0/(m+1) for i=-1,m and j=-1,n 
 *
 * REFERENCE: Slides 28, 24
 *
 * GLOBAL DATA:
 *  IN int start : begin of the local chunk
 *  IN int end1  : end+1 of the local chunk
 * USED FUNCTIONS: APPL: I2ij, Vec_Set_Value
 */
void Init_Boundary_Vec(vector_chunk b)
{
        int I, i, j;   double v, h;
 
        h = 1.0/(m+1);
        for (I=start; I<end1; I++) {
                I2ij(I, &i, &j);
                v = 0.0;
                if (i==0)   v = v + /*u(-1,j):*/  h * 0;     /* = 0 */
                if (i==m-1) v = v + /*u(m, j):*/  h * (m+1); /* = 1 */
                if (j==0)   v = v + /*u(i,-1):*/  h * (i+1);
                if (j==n-1) v = v + /*u(i, n):*/  h * (i+1);
                if (v != 0) Vec_Set_Value(b,I,v);
        }
}

/*------------------------------
 * void Init_Exact_Solution_Vec
 *  OUT vector_chunk u: the local chunk of vector u
 *
 * TASK: Store all boundary values in the vector memory 
 *       (that is already allocated).
 *       In this example, the inner values are given by:
 *       u(i,j) = h * (i+1)  with  h = 1.0/(m+1) for i=0..m-1 and j=0..n-1 
 *
 * REFERENCE: Slides 28, 24
 *
 * GLOBAL DATA:
 *  IN int start : begin of the local chunk
 *  IN int end1  : end+1 of the local chunk
 * USED FUNCTIONS: APPL: I2ij, Vec_Set_Value
 */
void Init_Exact_Solution_vec(vector_chunk u)
{
        int I, i, j;   double v, h;
 
        h = 1.0/(m+1);
        for (I=start; I<end1; I++) {
                I2ij(I, &i, &j);
                v = /*u(i, j):*/ h * (i+1); Vec_Set_Value(u,I,v);
        }
}
/*>>>>>>>>>>>>>>>>>>>>>> -end- of task 12 <<<<<<<<<<<<<<<<<<<<<<*/
