/* CG/solution/parts/cgp14.c */

/****************************************************************
 *                                                              *
 * This file has been written as a sample solution to an        *
 * exercise in a course given at the High Performance           *
 * Computing Centre Stuttgart (HLRS).                           *
 * It is made freely available with the understanding that      *
 * every copy of this file must include this header and that    *
 * HLRS takes no responsibility for the use of the enclosed     *
 * teaching material.                                           *
 *                                                              *
 * This software is partitioned into 16 tasks.                  *
 * Each task can be implemented by one group of the course      *
 * Files: CG/solution/cg_all.c - Solution                       *
 *        CG/skel/cg_01.c      - Skeletons that should be used  *
 *             .. cg_15.c        to implement each task         *
 *        CG/solution/cgp00.c  - Source part of each task       *
 *             ..     cgp16.c                                   *
 *                                                              *
 * Authors: Dr. Rolf Rabenseifner                               *
 *          Felix Triebel                                       *
 *          Michael Speck                                       *
 *          Felix Triebel                                       *
 *          Traugott Streicher                                  *
 *                                                              *
 * Contact: rabenseifner@hlrs.de                                *
 *                                                              *
 ****************************************************************/
                                                           /*EOH*/
/*>>>>>>>>>>>>>>>>>>>>>> begin of task 14 <<<<<<<<<<<<<<<<<<<<<<*/

/************************ print results ************************/

void recv_token(void)
{
#ifndef serial
        int dummy; 
        MPI_Status status;
        if (my_rank > 0)
                MPI_Recv(&dummy, 0, MPI_INT, my_rank-1, MSG_TAG, MPI_COMM_WORLD, &status);
#endif
}

void flush_and_send_token(void)
{
#ifndef serial
        MPI_Status status;
        int dummy; 
#endif
        fflush(stdout);
#ifndef serial
        MPI_Send(&dummy, 0, MPI_INT, (my_rank+1) % numprocs, MSG_TAG, MPI_COMM_WORLD);
        /* Process 0must wait until the last process (numprocs-1) has finished its output */ 
        if (my_rank == 0)
                MPI_Recv(&dummy, 0, MPI_INT, numprocs-1, MSG_TAG, MPI_COMM_WORLD, &status);
#endif
}

void Print_Decomp(void)
{
        int i,j, I, inew,jnew;
        if (my_rank == 0)
        {
                for (i=0; i<m; i++)
                {
                    printf("Decomp: i=%2d",i);
                    for (j=0; j<n; j++)
                    {
                        ij2I(i, j, &I);
                        I2ij(I, &inew, &jnew);
                        printf(" I(%1d,%1d)=%2d%c",inew,jnew,I, ((i==inew)&&(j==jnew) ? ' ' : '!') );
                    }
                    printf("\n");
                }
        }
        fflush(stdout);
}

void print_vec(char *name, vector_chunk v1)
{
        int I;
        recv_token();
        for (I=start; I<end1; I++)
                printf("%s[%2d] = %6.3f\n", name, I, v1[I-start]);
        flush_and_send_token();
}

void print_vec2d(char *name, vector_chunk v2, int mprint, int nprint)
{
        int I, rank;
        double* whole_v2;
#ifndef serial
        MPI_Status status;
#endif


        whole_v2 = (double*)save_malloc(sizeof(double)*n*m);
#ifndef serial
        if (my_rank > 0)
        {
                MPI_Send(v2, end1-start, MPI_DOUBLE, 0, MSG_TAG, MPI_COMM_WORLD);
        } else {
                for (rank=1; rank<numprocs; rank++)
                {
                        MPI_Recv(&(whole_v2[Rank2StartRow(rank)]), n*m, MPI_DOUBLE, 
                                 rank, MSG_TAG, MPI_COMM_WORLD, &status);
                }
        }
#endif
        for (I=start; I<end1; I++) whole_v2[I] = v2[I-start];
        appl_print_vec(name, whole_v2, mprint, nprint);
        free(whole_v2);
}

void print_mat(char *name, struct matrix_chunk *A)
{
        int I, k;
        recv_token();
        for (I=start; I<end1; I++)
        {
                printf("%s[%2d] = ", name, I);
                for (k=0; k<A->row[I-start].size; k++)
                        printf(" %1d(%1d):%2.0f", A->row[I-start].J[k], A->row[I-start].J_loc[k],
                                        A->row[I-start].value[k]);
                printf("\n");
        }
        flush_and_send_token();
}

void print_halo(void)
{
        int I_loc, halo_pos;
        recv_token();
        printf("Halo: my_rank =%3d - local index:", my_rank);
          for (I_loc=0; I_loc<end1-start;     I_loc++) printf(" %2d",I_loc);printf(" |");
          for (halo_pos=0; halo_pos<halo_info.size; halo_pos++) printf(" %2d",end1-start+halo_pos);printf("\n");

        printf("                   - orig. index:");
          for (I_loc=0; I_loc<end1-start;     I_loc++) printf(" %2d",start+I_loc);printf(" |");
          for (halo_pos=0; halo_pos<halo_info.size; halo_pos++) printf(" %2d",halo_info.I[halo_pos]);
          printf("\n");
        printf("                   - size=%d,  size_allocated=%d\n",
                          halo_info.size, halo_info.size_allocated);
        flush_and_send_token();
}

void print_vec_withhalo(char *name, vector_chunk v1)
{
        int I_loc, halo_pos;
        recv_token();
        printf("Halo: my_rank =%3d - local index:", my_rank);
          for (I_loc=0; I_loc<end1-start;     I_loc++) printf(" %3d",I_loc);printf(" |");
          for (halo_pos=0; halo_pos<halo_info.size; halo_pos++) printf(" %3d",end1-start+halo_pos);
          printf("\n");

        printf("                   - orig. index:");
          for (I_loc=0; I_loc<end1-start;     I_loc++) printf(" %3d",start+I_loc);printf(" |");
          for (halo_pos=0; halo_pos<halo_info.size; halo_pos++) printf(" %3d",halo_info.I[halo_pos]);
          printf("\n");
        printf("                   - vector %s:   ",name);
          for (I_loc=0; I_loc<end1-start; I_loc++) printf(" %3.1f",v1[I_loc]);printf(" |");
          for (I_loc=end1-start; I_loc<end1-start+halo_info.size; I_loc++) printf(" %3.1f",v1[I_loc]);
          printf("\n");
        printf("                   - size=%d,  size_allocated=%d\n",
                          halo_info.size, halo_info.size_allocated);
        flush_and_send_token();
}

void print_recv_info(void)
{
        int i, rank;
        recv_token();
        printf("Recv: my_rank =%3d - recv_num = %d, recv_ranks:",
                my_rank, recv_num);
        for (i=0; i<recv_num; i++) printf(" %2d",recv_ranks[i]);printf("\n");
        for (rank=0; rank<numprocs; rank++)
        {       printf("                   - recv_halo[%2d].size=%2d,",rank,recv_halo[rank].size);
                printf(" .I_loc:");
                for (i=0; i<recv_halo[rank].size; i++)
                        printf(" %2d",recv_halo[rank].I_loc[i]);
                printf("\n");
        }
        flush_and_send_token();
}


void print_send_info(void)
{
        int i, rank;
        recv_token();
        printf("Send: my_rank =%3d - send_num = %d, send_ranks:",
                my_rank, send_num);
        for (i=0; i<send_num; i++) printf(" %2d",send_ranks[i]);printf("\n");
        for (rank=0; rank<numprocs; rank++)
        {       printf("                   - send_halo[%2d].size=%2d,",rank,send_halo[rank].size);
                printf(" .I_loc:");
                for (i=0; i<send_halo[rank].size; i++)
                        printf(" %2d",send_halo[rank].I_loc[i]);
                printf("\n");
        }
        flush_and_send_token();
}

/*>>>>>>>>>>>>>>>>>>>>>> -end- of task 14 <<<<<<<<<<<<<<<<<<<<<<*/
